@@ -1,20 +1,20 @@
 # -*- coding: utf8 -*-
-#
-# High-level Python bindings build on top of the low-level
-# C API (clinkgrammar)
-# See http://www.abisource.com/projects/link-grammar/api/index.html to get
-# more information about C API
+"""
+High-level Python bindings build on top of the low-level
+C API (clinkgrammar)
+See http://www.abisource.com/projects/link-grammar/api/index.html to get
+more information about C API
+"""
 
-import sys
-
-# In Python3, importing just _clinkgrammar raises exception that the
-# module is not found.
 try:
-    import linkgrammar._clinkgrammar as clg
+    #pylint: disable=no-name-in-module
+    import linkgrammar.clinkgrammar as clg
 except ImportError:
-    import _clinkgrammar as clg
+    #pylint: import-error
+    import clinkgrammar as clg
 
-__all__ = ['ParseOptions', 'Dictionary', 'Link', 'Linkage', 'Sentence']
+Clinkgrammar = clg
+__all__ = ['ParseOptions', 'Dictionary', 'Link', 'Linkage', 'Sentence', 'Clinkgrammar']
 
 
 class ParseOptions(object):
@@ -50,10 +50,6 @@ class ParseOptions(object):
             clg.parse_options_delete(self._obj)
             del self._obj
 
-    @property
-    def version(self):
-        return clg.linkgrammar_get_version()
-
     @property
     def verbosity(self):
         """
@@ -66,8 +62,8 @@ class ParseOptions(object):
     def verbosity(self, value):
         if not isinstance(value, int):
             raise TypeError("Verbosity must be set to an integer")
-        if value not in (0, 1, 2, 3, 4, 5, 6):
-            raise ValueError("Verbosity levels can be any integer between 0 and 6 inclusive")
+        if value not in range(0,15):
+            raise ValueError("Verbosity levels can be any integer between 0 and 15 inclusive")
         clg.parse_options_set_verbosity(self._obj, value)
 
     @property
@@ -266,8 +262,12 @@ class Dictionary(object):
             clg.dictionary_delete(self._obj)
             del self._obj
 
-    def get_max_cost(self):
-        return clg.dictionary_get_max_cost(self._obj)
+    def linkgrammar_get_dict_version(self):
+        return clg.linkgrammar_get_dict_version(self._obj)
+
+    def linkgrammar_get_dict_locale(self):
+        return clg.linkgrammar_get_dict_locale(self._obj)
+
 
 class Link(object):
     def __init__(self, linkage, index, left_word, left_label, right_label, right_word):
@@ -302,7 +302,8 @@ class Link(object):
 class Linkage(object):
 
     def __init__(self, idx, sentence, parse_options):
-        self.sentence, self.parse_options = sentence, parse_options  # keep all args passed into clg.* fn
+        # Keep all args passed into clg.* functions.
+        self.sentence, self.parse_options = sentence, parse_options
         self._obj = clg.linkage_create(idx, sentence._obj, parse_options)
 
     def __del__(self):
@@ -310,8 +311,12 @@ class Linkage(object):
             clg.linkage_delete(self._obj)
             del self._obj
 
-    def null_count(self):
-        return clg.sentence_null_count(self._obj)
+    def __nonzero__(self):
+        """Return False for SAT sentinel value (NULL); else return True."""
+        return bool(self._obj)
+
+    __bool__ = __nonzero__      # Account python3
+
 
     def num_of_words(self):
         return clg.linkage_get_num_words(self._obj)
@@ -332,6 +337,9 @@ class Linkage(object):
     def link_cost(self):
         return clg.linkage_link_cost(self._obj)
 
+    def disjunct_cost(self):
+        return clg.linkage_disjunct_cost(self._obj)
+
     def link(self, i):
         return Link(self, i, self.word(clg.linkage_get_link_lword(self._obj, i)),
                     clg.linkage_get_link_llabel(self._obj, i),
@@ -371,12 +379,9 @@ class Sentence(object):
         for linkage in linkages:
             print linkage.diagram()
     """
-    text = None
-    dict = None
-    parse_options = None
-
-    def __init__(self, text, dict, parse_options):
-        self.text, self.dict, self.parse_options = text, dict, parse_options  # keep all args passed into clg.* fn
+    def __init__(self, text, lgdict, parse_options):
+        # Keep all args passed into clg.* functions.
+        self.text, self.dict, self.parse_options = text, lgdict, parse_options
         self._obj = clg.sentence_create(self.text, self.dict._obj)
 
     def __del__(self):
@@ -385,6 +390,7 @@ class Sentence(object):
             del self._obj
 
     def split(self, parse_options=None):
+        """Split a sentence. If an error occurs, return a negative number."""
         if not parse_options:
             parse_options = self.parse_options
         return clg.sentence_split(self._obj, parse_options._obj)
@@ -398,14 +404,28 @@ class Sentence(object):
     def num_linkages_post_processed(self):
         return clg.sentence_num_linkages_post_processed(self._obj)
 
+    def __len__(self):
+        """The number of tokens in the sentence."""
+        return clg.sentence_length(self._obj)
+
+    def null_count(self):
+        """Number of null links in the linkages of this sentence."""
+        return clg.sentence_null_count(self._obj)
+
     class sentence_parse(object):
         def __init__(self, sent):
             self.sent = sent
             self.num = 0
-            clg.sentence_parse(sent._obj, sent.parse_options._obj)
+            self.rc = clg.sentence_parse(sent._obj, sent.parse_options._obj)
+
+        def __nonzero__(self):
+            """Return False if there was a split or parse error; else return True."""
+            return self.rc >= 0
+
+        __bool__ = __nonzero__      # Account python3
 
         def __iter__(self):
-            if (0 == clg.sentence_num_valid_linkages(self.sent._obj)):
+            if 0 == clg.sentence_num_valid_linkages(self.sent._obj):
                 return iter(())
             return self
 
@@ -413,10 +433,12 @@ class Sentence(object):
             if self.num == clg.sentence_num_valid_linkages(self.sent._obj):
                 raise StopIteration()
             linkage = Linkage(self.num, self.sent, self.sent.parse_options._obj)
+            if not linkage:  # SAT sentinel value
+                raise StopIteration()
             self.num += 1
             return linkage
 
-        __next__=next      # Account python3
+        __next__= next              # Account python3
 
     def parse(self):
         return self.sentence_parse(self)
