@@ -74,8 +74,8 @@ Afdict_class * afdict_find(Dictionary afdict, const char * con, bool notify_err)
 	}
 	if (notify_err) {
 		prt_error("Warning: Unknown class name %s found near line %d of %s.\n"
-                "\tThis class name will be ignored.",
-                con, afdict->line_number, afdict->name);
+		          "\tThis class name will be ignored.",
+		          con, afdict->line_number, afdict->name);
 	}
 	return NULL;
 }
@@ -108,9 +108,9 @@ static void load_affix(Dictionary afdict, Dict_node *dn, int l)
 		{
 			/* ??? should we support here more than one class? */
 			prt_error("Warning: Word \"%s\" found near line %d of %s.\n"
-					  "\tWord has more than one connector.\n"
-					  "\tThis word will be ignored.",
-					  dn->string, afdict->line_number, afdict->name);
+			          "\tWord has more than one connector.\n"
+			          "\tThis word will be ignored.",
+			          dn->string, afdict->line_number, afdict->name);
 			return;
 		}
 
@@ -174,7 +174,7 @@ static void get_dict_affixes(Dictionary dict, Dict_node * dn,
 	if (w_len > MAX_WORD)
 	{
 		prt_error("Error: word '%s' too long (%zd), program may malfunction\n",
-				w, w_len);
+		          w, w_len);
 		w_len = MAX_WORD;
 	}
 	/* (strlen(w_last) can be cached for speedup) */
@@ -201,59 +201,36 @@ static void get_dict_affixes(Dictionary dict, Dict_node * dn,
 }
 
 /**
- * Convert a list of utf8 chars to wide-chars.  The reason for doing
- * this is kind-of dorky: its so that we can easily find,
- * character-by-character, if a given character is a quotation mark
- * or a bullet.  This works only because the quotation marks and
- * bullets are exactly one (wide) character in length. I would like
- * it better if we didn't do this wide-char conversion, since wide-chars
- * are badly-behaved in crazy locales, and on MS Windows.
+ * Concatenate the definitions for the given affix class.
+ * This allows specifying the characters in different definitions
+ * instead in a one long string, e.g. instead of:
+ * ""«»《》【】『』`„": QUOTES+;
+ * One can specify (note the added spaces):
+ * """  «»  《》 【】 『』  ` „: QUOTES+;
+ * Or even:
+ * """: QUOTES+;
+ * «» : QUOTES+;
+ * etc.
+ * Note that if there are no definitions or only one definition, there is
+ * nothing to do.
+ * The result is written to the first entry.
+ * @param classno The given affix class.
  */
-static bool afdict_to_wide(Dictionary afdict, int classno)
+static void concat_class(Dictionary afdict, int classno)
 {
 	Afdict_class * ac;
-	wchar_t * wqs;
-	mbstate_t mbs;
 	size_t i;
-	size_t w;
 	dyn_str * qs;
-	const char *pqs;
 
 	ac = AFCLASS(afdict, classno);
-	if (0 == ac->length) return true;
+	if (1 >= ac->length) return;
 
 	qs = dyn_str_new();
 	for (i = 0; i < ac->length; i++)
 		dyn_strcat(qs, ac->string[i]);
 
-	/*
-	 * Convert utf8 to wide chars before use.
-	 * In case of error the result is undefined.
-	 */
-	pqs = qs->str;
-	memset(&mbs, 0, sizeof(mbs));
-	w = mbsrtowcs(NULL, &pqs, 0, &mbs);
-	if (0 > (int) w) /* mbsrtowcs returns ((size_t) -1) on error */
-	{
-		prt_error("Error: Affix dictionary: %s: "
-		          "Invalid utf8 character\n", afdict_classname[classno]);
-		return false;
-	}
-
-	/* Store the wide char version at the AFCLASS entry. */
-	if (ac->mem_elems <= w)
-	{
-		ac->mem_elems = w + 1;
-		ac->string = realloc(ac->string, sizeof(*wqs) * ac->mem_elems);
-	}
-	wqs = (wchar_t *)ac->string;
-	pqs = qs->str;
-	(void)mbsrtowcs(wqs, &pqs, w, &mbs);
-	wqs[w] = L'\0';
-
+	ac->string[0] = string_set_add(qs->str, afdict->string_set);
 	dyn_str_delete(qs);
-
-	return true;
 }
 
 /* Compare lengths of strings, for qsort */
@@ -271,6 +248,7 @@ static int cmplen(const void *a, const void *b)
  * needed - especially to first free memory and initialize the affix dict
  * structure.).
  */
+#define D_AI 11
 static bool afdict_init(Dictionary dict)
 {
 	Afdict_class * ac;
@@ -300,8 +278,8 @@ static bool afdict_init(Dictionary dict)
 	if ((1 < ac->length) || ((1 == ac->length) && (1 != strlen(ac->string[0]))))
 	{
 		prt_error("Error: afdict_init: Invalid value for class %s in file %s"
-					 " (should have been one ASCII punctuation - ignored)\n",
-					  afdict_classname[AFDICT_INFIXMARK], afdict->name);
+		          " (should have been one ASCII punctuation - ignored)\n",
+		          afdict_classname[AFDICT_INFIXMARK], afdict->name);
 		free((void *)ac->string);
 		ac->length = 0;
 		ac->mem_elems = 0;
@@ -324,7 +302,7 @@ static bool afdict_init(Dictionary dict)
 		affix_list_add(afdict, &afdict->afdict_class[AFDICT_INFIXMARK], "");
 	}
 
-	if (0 || 4 < verbosity) /* debug - !verbosity is not set so early for now */
+	if (debug_level(+D_AI))
 	{
 		size_t l;
 
@@ -333,12 +311,13 @@ static bool afdict_init(Dictionary dict)
 		{
 				if (0 == ac->length) continue;
 				lgdebug(+0, "Class %s, %zd items:",
-						  afdict_classname[ac-afdict->afdict_class], ac->length);
+				        afdict_classname[ac-afdict->afdict_class], ac->length);
 				for (l = 0; l < ac->length; l++)
 					lgdebug(0, " '%s'", ac->string[l]);
 				lgdebug(0, "\n");
 		}
 	}
+#undef D_AI
 
 	/* Store the SANEMORPHISM regex in the unused (up to now)
 	 * regex_root element of the affix dictionary, and precompile it */
@@ -407,8 +386,8 @@ static bool afdict_init(Dictionary dict)
 	}
 #endif /* AFDICT_ORDER_NOT_PRESERVED */
 
-	if (! afdict_to_wide(afdict, AFDICT_QUOTES)) return false;
-	if (! afdict_to_wide(afdict, AFDICT_BULLETS)) return false;
+	concat_class(afdict, AFDICT_QUOTES);
+	concat_class(afdict, AFDICT_BULLETS);
 
 	if (! anysplit_init(afdict)) return false;
 
@@ -437,6 +416,7 @@ dictionary_six(const char * lang, const char * dict_name,
  * Read dictionary entries from a wide-character string "input".
  * All other parts are read from files.
  */
+#define D_DICT 10
 static Dictionary
 dictionary_six_str(const char * lang,
                    const char * input,
@@ -456,9 +436,7 @@ dictionary_six_str(const char * lang,
 	t = strrchr (lang, '/');
 	t = (NULL == t) ? lang : t+1;
 	dict->lang = string_set_add(t, dict->string_set);
-#ifdef _DEBUG
-	prt_error("Info: Language: %s\n", dict->lang);
-#endif
+	lgdebug(D_USER_FILES, "Debug: Language: %s\n", dict->lang);
 	dict->name = string_set_add(dict_name, dict->string_set);
 
 	/*
@@ -527,6 +505,36 @@ dictionary_six_str(const char * lang,
 		return dict;
 	}
 
+	/* If we don't have a locale per dictionary, the following
+	 * will also set the program's locale. */
+	dict->locale = linkgrammar_get_dict_locale(dict);
+	set_utf8_program_locale();
+
+#ifdef HAVE_LOCALE_T
+	/* We have a locale per dictionary. */
+	if (NULL != dict->locale)
+		dict->locale_t = newlocale_LC_CTYPE(dict->locale);
+
+	/* If we didn't succeed to set the dictionary locale, the program will
+	 * SEGFAULT when it tries to use it with the isw*() functions.
+	 * So set it to the current program's locale as a last resort. */
+	if (NULL == dict->locale)
+	{
+		dict->locale = setlocale(LC_CTYPE, NULL);
+		dict->locale_t = newlocale_LC_CTYPE(setlocale(LC_CTYPE, NULL));
+		prt_error("Warning: Couldn't set dictionary locale! "
+		          "Using current program locale %s", dict->locale);
+	}
+	/* If dict->locale is still not set, there is a bug. */
+	assert((locale_t)0 != dict->locale_t, "Dictionary locale is not set.");
+#else
+	/* We don't have a locale per dictionary - but anyway make sure
+	 * dict->locale is consistent with the current program's locale,
+	 * and especially that it is not NULL. It still indicates the intended
+	 * locale of this dictionary and the locale of the compiled regexs. */
+	dict->locale = setlocale(LC_CTYPE, NULL);
+#endif /* HAVE_LOCALE_T */
+
 	dict->affix_table = dictionary_six(lang, affix_name, NULL, NULL, NULL, NULL);
 	if (dict->affix_table == NULL)
 	{
@@ -536,8 +544,25 @@ dictionary_six_str(const char * lang,
 	if (! afdict_init(dict))
 		goto failure;
 
+	/*
+	 * Process the regex file.
+	 * We have to compile regexs using the dictionary locale,
+	 * so make a temporary locale swap.
+	 */
 	if (read_regex_file(dict, regex_name)) goto failure;
-	if (compile_regexs(dict->regex_root, dict)) goto failure;
+
+	const char *locale = setlocale(LC_CTYPE, NULL);
+	locale = strdupa(locale); /* setlocale() uses static memory. */
+	setlocale(LC_CTYPE, dict->locale);
+	lgdebug(+D_DICT, "Regexs locale %s\n", setlocale(LC_CTYPE, NULL));
+
+	if (compile_regexs(dict->regex_root, dict))
+	{
+		locale = setlocale(LC_CTYPE, locale);
+		goto failure;
+	}
+	locale = setlocale(LC_CTYPE, locale);
+	assert(NULL != locale, "Cannot restore program locale\n");
 
 #ifdef USE_CORPUS
 	dict->corpus = lg_corpus_new();
@@ -587,7 +612,7 @@ dictionary_six(const char * lang, const char * dict_name,
 	}
 
 	dict = dictionary_six_str(lang, input, dict_name, pp_name,
-				  cons_name, affix_name, regex_name);
+	                          cons_name, affix_name, regex_name);
 
 	free(input);
 	return dict;
