@@ -23,6 +23,8 @@
 #include "string-set.h"
 #include "word-utils.h"
 
+#define D_PRUNE 5
+
 #define CONTABSZ 8192
 typedef Connector * connector_table;
 
@@ -674,7 +676,7 @@ DBG(printf("after purging: "); print_expression(x->exp); printf("\n"););
 			}
 		}
 
-		if (verbosity > 2)
+		if (debug_level(D_PRUNE))
 		{
 			printf("l->r pass removed %d\n", N_deleted);
 			print_expression_sizes(sent);
@@ -710,7 +712,7 @@ DBG(printf("after purging: "); print_expression(x->exp); printf("\n"););
 			}
 		}
 
-		if (verbosity > 2)
+		if (debug_level(D_PRUNE))
 		{
 			printf("r->l pass removed %d\n", N_deleted);
 			print_expression_sizes(sent);
@@ -1212,7 +1214,7 @@ int power_prune(Sentence sent, Parse_Options opts)
 			}
 			sent->word[w].d = nd;
 		}
-		if (verbosity > 2)
+		if (debug_level(D_PRUNE))
 		{
 			printf("l->r pass changed %d and deleted %zu\n", pc->N_changed, N_deleted);
 		}
@@ -1248,7 +1250,7 @@ int power_prune(Sentence sent, Parse_Options opts)
 			sent->word[w].d = nd;
 		}
 
-		if (verbosity > 2)
+		if (debug_level(D_PRUNE))
 		{
 			printf("r->l pass changed %d and deleted %zu\n",
 				pc->N_changed, N_deleted);
@@ -1262,10 +1264,10 @@ int power_prune(Sentence sent, Parse_Options opts)
 	pt = NULL;
 	pc->pt = NULL;
 
-	if (verbosity >= 2) printf("power prune cost: %d\n", pc->power_cost);
+	if (debug_level(D_PRUNE)) printf("power prune cost: %d\n", pc->power_cost);
 
 	print_time(opts, "power pruned");
-	if (verbosity > 2)
+	if (debug_level(D_PRUNE))
 	{
 		printf("\nAfter power_pruning:\n");
 		print_disjunct_counts(sent);
@@ -1607,15 +1609,12 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 			}
 		}
 
-		if (verbosity > 2)
-		{
-			printf("pp_prune pass deleted %d\n", N_deleted);
-		}
+		if (debug_level(D_PRUNE)) printf("pp_prune pass deleted %d\n", N_deleted);
 	}
 	delete_unmarked_disjuncts(sent);
 	cms_table_delete(cmt);
 
-	if (verbosity > 2)
+	if (debug_level(D_PRUNE))
 	{
 		printf("\nAfter pp_pruning:\n");
 		print_disjunct_counts(sent);
