@@ -13,38 +13,31 @@
 #ifndef _LINK_GRAMMAR_UTILITIES_H_
 #define _LINK_GRAMMAR_UTILITIES_H_
 
-#ifdef __CYGWIN__
-#define _WIN32 1
-#endif /* __CYGWIN__ */
-
-#ifndef _WIN32
-#include <langinfo.h>
-#endif
+/* The _Win32 definitions are for native-Windows compilers.
+ * This includes MSVC (only version >=14 is supported) and MINGW (under
+ * MSYS or Cygwin). The _WIN32 definitions are not for Cygwin, which
+ * doesn't define _WIN32.
+ * FIXME: MinGW still has problems with UTF-8 output.
+ */
 
 #include <ctype.h>
 #include <stdio.h>
+#ifdef _WIN32
+#define _CRT_RAND_S
+#endif /* _WIN32 */
 #include <stdlib.h>
 #include <string.h>
-
-#ifndef __CYGWIN__
-/* I was told that cygwin does not have these files. */
 #include <wchar.h>
 #include <wctype.h>
-#endif
-
-#if defined(__CYGWIN__) && defined(__MINGW32__)
-/* Some users have CygWin and MinGW installed!
- * In this case, use the MinGW versions of UTF-8 support. */
-#include <wchar.h>
-#include <wctype.h>
-#endif
 
 #include "error.h"
 
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
 #elif defined __GNUC__
+#ifndef alloca
 # define alloca __builtin_alloca
+#endif /* !alloca */
 #elif defined _AIX
 # define alloca __alloca
 #elif defined _MSC_VER
@@ -58,54 +51,62 @@ extern "C"
 void *alloca (size_t);
 #endif
 
+#ifndef strdupa
+/* In the following, the argument should not have side effects. */
+#define strdupa(s) strcpy(alloca(strlen(s)+1), s)
+#endif
+
+#ifdef _MSC_VER
+/* These definitions are incorrect, as these functions are different(!)
+ * (non-standard functionality).
+ * See http://stackoverflow.com/questions/27754492 . Fortunately,
+ * MSVC 14 supports C99 functions so these definitions are now unneeded.
+ * (LG library compilation is unsupported for previous MSVC versions.)
+ * (Left here for documentation.) */
+#if 0
+/* MS Visual C uses non-standard string function names */
+#define snprintf _snprintf
+#define vsnprintf _vsnprintf
+#endif
+
+/* Avoid plenty of: warning C4090: 'function': different 'const' qualifiers.
+ * This happens, for example, when the argument is "const void **". */
+#define free(x) free((void *)x)
+#define realloc(x, s) realloc((void *)x, s)
+#define memcpy(x, y, s) memcpy((void *)x, (void *)y, s)
+#define qsort(x, y, z, w) qsort((void *)x, y, z, w)
+#endif /* _MSC_VER */
+
 #ifdef _WIN32
 #include <windows.h>
 #include <mbctype.h>
 
-#ifdef _MSC_VER
-/* The Microsoft Visual C compiler doesn't support the "inline" keyword.
- * But it does support _inline as a synonym... */
-#define inline _inline
-
-/* MS Visual C does not have any function normally found in strings.h */
-/* In particular, be careful to avoid including strings.h */
-
-/* MS Visual C uses non-standard string function names */
-#define snprintf _snprintf
-#define vsnprintf _vsnprintf
-#define strcasecmp _stricmp
-#ifndef strdup
-#define strdup _strdup
-#endif
+#ifndef strncasecmp
 #define strncasecmp(a,b,s) strnicmp((a),(b),(s))
-
-/* Microsoft does not support %zu -- it wants %Iu */
-int printf_compat(const char *, ...);
-int fprintf_compat(FILE *, const char *, ...);
-int sprintf_compat(char *, const char *, ...);
-
-#define fprintf fprintf_compat
-#define printf printf_compat
-#define sprintf sprintf_compat
-
-/* MS Visual C does not support some C99 standard floating-point functions */
-#define fmaxf(a,b) ((a) > (b) ? (a) : (b))
-#define floorf(x) floor(x)
-#define ceilf(x) ceil(x)
+#endif
 
 /* MS changed the name of rand_r to rand_s */
 #define rand_r(seedp) rand_s(seedp)
+/* And strtok_r is strtok_s */
+#define strtok_r strtok_s
 
-#endif /* _MSC_VER */
+/* Native windows has locale_t, and hence HAVE_LOCALE_T is defined here.
+ * However, MinGW currently doesn't have locale_t. If/when it has locale_t,
+ * "configure" will define HAVE_LOCALE_T for it. */
+#ifndef __MINGW32__
+#define HAVE_LOCALE_T
+#endif
 
-/* Apparently, MinGW is also missing a variety of standard functions.
- * Not surprising, since MinGW is intended for compiling Windows
- * programs on Windows.
- * MINGW is also known as MSYS */
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
-/* strtok_r() is missing in Windows */
-char * strtok_r (char *s, const char *delim, char **saveptr);
+#ifdef HAVE_LOCALE_T
+#define locale_t _locale_t
+#define iswupper_l  _iswupper_l
+#define iswalpha_l  _iswalpha_l
+#define iswdigit_l  _iswdigit_l
+#define iswspace_l  _iswspace_l
+#define towlower_l  _towlower_l
+#define towupper_l  _towupper_l
+#define freelocale _free_locale
+#endif /* HAVE_LOCALE_T */
 
 /* strndup() is missing in Windows. */
 char * strndup (const char *str, size_t size);
@@ -114,54 +115,21 @@ char * strndup (const char *str, size_t size);
  * cygwin just doesn't work very well. So we use our own custom version,
  * instead.
  */
-link_public_api(size_t) lg_mbrtowc(wchar_t *, const char *, size_t, mbstate_t *);
 #ifdef mbrtowc
 #undef mbrtowc
 #endif
+size_t lg_mbrtowc(wchar_t *, const char *, size_t n, mbstate_t *ps);
 #define mbrtowc(w,s,n,x) lg_mbrtowc(w,s,n,x)
-#endif /* _MSC_VER || __MINGW32__ */
-
-/*
- * CYGWIN prior to version 1.7 did not have UTF8 support, or wide
- * chars ... However, MS Visual C does, as does MinGW.  Since
- * some users have both cygwin and MinGW installed, crap out the
- * UTF8 code only when MinGW is missing (and the CYGWIN version
- * is very old) XXX This code is dangerous and should be removed.
- */
-#if defined (__CYGWIN__) && !defined(__MINGW32__)
-#if CYGWIN_VERSION_DLL_MAJOR < 1007
-#error "Your Cygwin version is too old! Version 1.7 or later is needed for UTF8 support!"
-#define mbstate_t char
-#define mbrtowc(w,s,n,x)  ({*((char *)(w)) = *(s); 1;})
-#define wcrtomb(s,w,x)    ({*((char *)(s)) = ((char)(w)); 1;})
-#define mbrlen(s,n,m)     (1)
-#define iswupper  isupper
-#define iswalpha  isalpha
-#define iswdigit  isdigit
-#define iswspace  isspace
-#define wchar_t   char
-#define towlower  tolower
-#define towupper  toupper
-#endif /* CYGWIN_VERSION_DLL_MAJOR < 1007 */
-#endif /* __CYGWIN__ and not __MINGW32__ */
-
 #endif /* _WIN32 */
 
 /* MSVC isspace asserts in debug mode, and mingw sometime returns true,
  * when passed utf8. Thus, limit to 7 bits for windows. */
-#if defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef _WIN32
   #define lg_isspace(c) ((0 < c) && (c < 127) && isspace(c))
 #else
   #define lg_isspace isspace
 #endif
 
-
-#if __APPLE__
-/* Junk, to keep the Mac OSX linker happy, because this is listed in
- * the link-grammar.def symbol export file.  */
-void lg_mbrtowc(void);
-#endif
-
 #if __APPLE__
 /* It appears that fgetc on Mac OS 10.11.3 "El Capitan" has a weird
  * or broken version of fgetc() that flubs reads of utf8 chars when
@@ -197,28 +165,38 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
    (Or in linux when I compile with -ansi) */
 #endif
 
+/* Cygwin < 2.6.0 doesn't have locale_t. */
+#ifdef HAVE_LOCALE_T
+locale_t newlocale_LC_CTYPE(const char *);
+#else
+typedef int locale_t;
+#define iswupper_l(c, l) iswupper(c)
+#define iswalpha_l(c, l) iswalpha(c)
+#define iswdigit_l(c, l) iswdigit(c)
+#define iswspace_l(c, l) iswspace(c)
+#define towlower_l(c, l) towlower(c)
+#define towupper_l(c, l) towupper(c)
+#define freelocale(l)
+#endif /* HAVE_LOCALE_T */
+
 #define STR(x) #x
 #define STRINGIFY(x) STR(x)
 
 #define FILELINE __FILE__ ":" STRINGIFY(__LINE__)
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-#define assert(ex, ...) {                                                   \
-	if (!(ex)) {                                                             \
-		prt_error("\nAssertion (" #ex ") failed at " FILELINE ": " __VA_ARGS__);  \
-		fprintf(stderr, "\n");                                                \
-		*((volatile int*) 0x0) = 42;  /* leave stack trace in debugger */     \
-	}                                                                        \
-}
+#ifdef _WIN32
+#define DEBUG_TRAP (*((volatile int*) 0x0) = 42)
 #else
+#define DEBUG_TRAP __builtin_trap()
+#endif
+
 #define assert(ex, ...) {                                                   \
 	if (!(ex)) {                                                             \
 		prt_error("\nAssertion (" #ex ") failed at " FILELINE ": " __VA_ARGS__);  \
 		fprintf(stderr, "\n");                                                \
-		__builtin_trap();  /* leave stack trace in debugger */                \
+		DEBUG_TRAP;  /* leave stack trace in debugger */                      \
 	}                                                                        \
 }
-#endif
 
 #if defined(__UCLIBC__)
 #define fmaxf(a,b) ((a) > (b) ? (a) : (b))
@@ -252,11 +230,11 @@ static inline size_t utf8_strlen(const char *s)
 {
 	mbstate_t mbss;
 	memset(&mbss, 0, sizeof(mbss));
-#if defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef _WIN32
 	return MultiByteToWideChar(CP_UTF8, 0, s, -1, NULL, 0)-1;
 #else
 	return mbsrtowcs(NULL, &s, 0, &mbss);
-#endif
+#endif /* _WIN32 */
 }
 
 /**
@@ -265,7 +243,7 @@ static inline size_t utf8_strlen(const char *s)
  */
 static inline size_t utf8_next(const char *s)
 {
-#ifdef _MSC_VER
+#ifdef _WIN32
 	/* mbrlen does not work correctly on Windows. See issue #285 */
 	/* https://github.com/opencog/link-grammar/issues/285 */
 	size_t len = 0;
@@ -286,10 +264,10 @@ static inline size_t utf8_next(const char *s)
 		return 1;
 	}
 	return len;
-#endif
+#endif /* _WIN32 */
 }
 
-static inline int is_utf8_upper(const char *s)
+static inline int is_utf8_upper(const char *s, locale_t dict_locale)
 {
 	mbstate_t mbs;
 	wchar_t c;
@@ -298,11 +276,11 @@ static inline int is_utf8_upper(const char *s)
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 	if (nbytes < 0) return 0;  /* invalid mb sequence */
-	if (iswupper(c)) return nbytes;
+	if (iswupper_l(c, dict_locale)) return nbytes;
 	return 0;
 }
 
-static inline int is_utf8_alpha(const char *s)
+static inline int is_utf8_alpha(const char *s, locale_t dict_locale)
 {
 	mbstate_t mbs;
 	wchar_t c;
@@ -311,11 +289,11 @@ static inline int is_utf8_alpha(const char *s)
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 	if (nbytes < 0) return 0;  /* invalid mb sequence */
-	if (iswalpha(c)) return nbytes;
+	if (iswalpha_l(c, dict_locale)) return nbytes;
 	return 0;
 }
 
-static inline int is_utf8_digit(const char *s)
+static inline int is_utf8_digit(const char *s, locale_t dict_locale)
 {
 	mbstate_t mbs;
 	wchar_t c;
@@ -324,11 +302,11 @@ static inline int is_utf8_digit(const char *s)
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 	if (nbytes < 0) return 0;  /* invalid mb sequence */
-	if (iswdigit(c)) return nbytes;
+	if (iswdigit_l(c, dict_locale)) return nbytes;
 	return 0;
 }
 
-static inline int is_utf8_space(const char *s)
+static inline int is_utf8_space(const char *s, locale_t dict_locale)
 {
 	mbstate_t mbs;
 	wchar_t c;
@@ -337,7 +315,7 @@ static inline int is_utf8_space(const char *s)
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 	if (nbytes < 0) return 0;  /* invalid mb sequence */
-	if (iswspace(c)) return nbytes;
+	if (iswspace_l(c, dict_locale)) return nbytes;
 
 	/* 0xc2 0xa0 is U+00A0, c2 a0, NO-BREAK SPACE */
 	/* For some reason, iswspace doesn't get this */
@@ -346,13 +324,14 @@ static inline int is_utf8_space(const char *s)
 	return 0;
 }
 
-static inline const char * skip_utf8_upper(const char * s)
+#if 0 /* Not in use. */
+static inline const char * skip_utf8_upper(const char * s, locale_t dict_locale)
 {
-	int nb = is_utf8_upper(s);
+	int nb = is_utf8_upper(s, dict_locale);
 	while (nb)
 	{
 		s += nb;
-		nb = is_utf8_upper(s);
+		nb = is_utf8_upper(s, dict_locale);
 	}
 	return s;
 }
@@ -362,7 +341,8 @@ static inline const char * skip_utf8_upper(const char * s)
  * two input strings match. Comparison stops when
  * both strings descend to lowercase.
  */
-static inline bool utf8_upper_match(const char * s, const char * t)
+static inline bool utf8_upper_match(const char * s, const char * t,
+                                    locale_t dict_locale)
 {
 	mbstate_t mbs, mbt;
 	wchar_t ws, wt;
@@ -374,7 +354,7 @@ static inline bool utf8_upper_match(const char * s, const char * t)
 	ns = mbrtowc(&ws, s, MB_CUR_MAX, &mbs);
 	nt = mbrtowc(&wt, t, MB_CUR_MAX, &mbt);
 	if (ns < 0 || nt < 0) return false;  /* invalid mb sequence */
-	while (iswupper(ws) || iswupper(wt))
+	while (iswupper_l(ws, dict_locale) || iswupper_l(wt, dict_locale))
 	{
 		if (ws != wt) return false;
 		s += ns;
@@ -385,9 +365,11 @@ static inline bool utf8_upper_match(const char * s, const char * t)
 	}
 	return true;
 }
+#endif /* Not in use. */
 
-void downcase_utf8_str(char *to, const char * from, size_t usize);
-void upcase_utf8_str(char *to, const char * from, size_t usize);
+void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
+void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
+int utf8_charlen(const char *);
 
 size_t lg_strlcpy(char * dest, const char *src, size_t size);
 void safe_strcpy(char *u, const char * v, size_t usize);
@@ -415,15 +397,6 @@ void * xalloc(size_t) GNUC_MALLOC;
 void * exalloc(size_t) GNUC_MALLOC;
 
 /* Tracking the space usage can help with debugging */
-#if defined(_WIN32) || __APPLE__
-  /* **MUST** define for win32, Mac OSX, because xfree is listed in
-   * link-grammar.def and the win/osx linker fails if there is no
-   * xfree. So, keep the linker happy. (xfree is used in both dict-file
-   * and sat solver.)
-   */
-  #define TRACK_SPACE_USAGE 1
-#endif /* defined(_WIN32) || __APPLE__ */
-
 #ifdef TRACK_SPACE_USAGE
 void xfree(void *, size_t);
 void exfree(void *, size_t);
@@ -446,6 +419,8 @@ void * object_open(const char *filename,
 
 bool file_exists(const char * dict_name);
 char * get_file_contents(const char *filename);
+void set_utf8_program_locale(void);
+bool try_locale(const char *);
 
 /**
  * Returns the smallest power of two that is at least i and at least 1
@@ -457,4 +432,4 @@ static inline unsigned int next_power_of_two_up(unsigned int i)
    return j;
 }
 
-#endif
+#endif /* _LINK_GRAMMAR_UTILITIES_H_ */
