@@ -15,12 +15,8 @@
  */
 
 #include <math.h>
-#include <stdio.h>
-#include <stdint.h>
 
-#include "count.h"
 #include "dict-api.h"
-#include "disjunct-utils.h"
 #include "string-set.h"
 #include "word-utils.h"
 
@@ -337,28 +333,53 @@ bool word_has_connector(Dict_node * dn, const char * cs, char direction)
 }
 #else /* CRAZY_OBESE_CHECKING_AGLO */
 
-static bool exp_has_connector(Exp * e, const char * cs, char direction)
+/**
+ * Return true if the given expression has the given connector.
+ * The connector cs argument must originally be in the dictionary string set.
+ */
+static bool exp_has_connector(const Exp * e, int depth, const char * cs,
+                              char direction, bool smart_match)
 {
 	E_list * el;
 	if (e->type == CONNECTOR_type)
 	{
-		if (direction == e->dir)
-			return easy_match(e->u.string, cs);
-		return false;
+		if (direction != e->dir) return false;
+		return smart_match ? easy_match(e->u.string, cs)
+		                   : string_set_cmp(e->u.string, cs);
 	}
+
+	if (depth == 0) return false;
+	if (depth > 0) depth--;
+
 	for (el = e->u.l; el != NULL; el = el->next)
 	{
-		if (exp_has_connector(el->e, cs, direction)) return false;
+		if (exp_has_connector(el->e, depth, cs, direction, smart_match))
+			return true;
 	}
 	return false;
 }
 
 bool word_has_connector(Dict_node * dn, const char * cs, char direction)
 {
-	return exp_has_connector(dn->exp, cs, direction);
+	return exp_has_connector(dn->exp, -1, cs, direction, /*smart_match*/true);
 }
 #endif /* CRAZY_OBESE_CHECKING_AGLO */
 
+/**
+ * Find if an expression has a connector ZZZ- (that an empty-word has).
+ * This is a costly way to find it. To reduce the overhead, the
+ * exp_has_connector() "depth" argument limits the expression depth check,
+ * supposing the ZZZ- connectors are not deep in the word expression.
+ * FIXME? A cheaper way is to have a dictionary entry which lists such
+ * words, or to mark such words at dictionary read time.
+ **/
+bool is_exp_like_empty_word(Dictionary dict, Exp *exp)
+{
+	const char *cs = string_set_lookup(EMPTY_CONNECTOR, dict->string_set);
+	if (NULL == cs) return false;
+	return exp_has_connector(exp, 2, cs, '-', /*smart_match*/false);
+}
+
 /**
  * If word has a connector, return it.
  * If word has more than one connector, return NULL.
